@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2025, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -35,6 +35,10 @@
 
 DEFAULT_LOG_DOMAIN("home");
 
+#if defined(__APPLE__) || defined(_WIN64)
+#define HAS_MIGRATION_ASSISTANT 1
+#endif
+
 using namespace base;
 using namespace mforms;
 
@@ -43,6 +47,12 @@ using namespace mforms;
 class mforms::ConnectionEntry : public base::Accessible {
   friend class ConnectionsSection;
 
+private:
+  //cairo_surface_t *_mbNewSticker;
+  //bool _showNewSticker = false;
+  cairo_surface_t *_migrationButton;
+  base::Rect _migrationButtonBounds;
+
 public:
   std::string connectionId;
 
@@ -129,6 +139,12 @@ protected:
 
   //--------------------------------------------------------------------------------------------------------------------
 
+  base::Rect getMigrationButtonBounds() {
+    return _migrationButtonBounds;
+  }
+
+  //--------------------------------------------------------------------------------------------------------------------
+
   /**
    * Draws the icon followed by the given text. The given position is that of the upper left corner
    * of the image.
@@ -165,6 +181,18 @@ public:
 
   ConnectionEntry(ConnectionsSection *aowner) : owner(aowner), compute_strings(false) {
     draw_info_tab = true;
+#ifdef HAS_MIGRATION_ASSISTANT
+    _migrationButton = Utilities::load_icon("migration_btn.png", true);
+#else
+    _migrationButton = nullptr;
+#endif
+  }
+
+  //--------------------------------------------------------------------------------------------------------------------
+
+  virtual ~ConnectionEntry() {
+    if (_migrationButton)
+      deleteSurface(_migrationButton);
   }
 
   //--------------------------------------------------------------------------------------------------------------------
@@ -231,7 +259,7 @@ public:
 
   //--------------------------------------------------------------------------------------------------------------------
 
-  virtual void draw_tile(cairo_t *cr, bool hot, double alpha, bool for_dragging) {
+  virtual void draw_tile(cairo_t *cr, bool hot, double alpha, bool for_dragging, bool on_migration = false) {
     base::Color titleColor = getTitleColor();
     base::Rect bounds = this->bounds;
     if (for_dragging)
@@ -274,6 +302,33 @@ public:
     cairo_show_text(cr, title.c_str());
     cairo_stroke(cr);
 
+    // Display _migrationButton in the bottom right corner of this->bounds
+    if (_migrationButton) {
+      auto btn_size = Utilities::getImageSize(_migrationButton);
+      double btn_scale = 0.8;
+      double btn_width = btn_size.width * btn_scale;
+      double btn_height = btn_size.height * btn_scale;
+      double btn_x = bounds.right() - btn_width - 5;
+      double btn_y = bounds.bottom() - btn_height - 5;
+      _migrationButtonBounds = base::Rect(btn_x, btn_y, btn_width, btn_height); 
+      cairo_save(cr);
+      cairo_translate(cr, btn_x, btn_y);
+      cairo_scale(cr, btn_scale, btn_scale);
+      cairo_set_source_surface(cr, _migrationButton, 0, 0);
+
+      float image_alpha =  0.25;
+      if (hot) {
+        image_alpha += 0.25;
+
+        if (on_migration) {
+          image_alpha += 0.5;
+        }
+      }
+
+      cairo_paint_with_alpha(cr, image_alpha * alpha);
+      cairo_restore(cr);
+    }
+
     cairo_set_font_size(cr, mforms::HomeScreenSettings::HOME_SMALL_INFO_FONT_SIZE);
 
     draw_tile_text(cr, x, y, alpha);
@@ -304,8 +359,12 @@ public:
 
   //--------------------------------------------------------------------------------------------------------------------
 
-  virtual void activate() {
-    owner->_owner->trigger_callback(HomeScreenAction::ActionOpenConnectionFromList, connectionId);
+  virtual void activate(int x = -1, int y = -1) {
+    if (x > 0 && y > 0 && _migrationButtonBounds.contains(x, y)) {
+      owner->_owner->trigger_callback(HomeScreenAction::ActionOpenMigrationAssistant, connectionId);
+    } else {
+      owner->_owner->trigger_callback(HomeScreenAction::ActionOpenConnectionFromList, connectionId);
+    }
   }
 
   //--------------------------------------------------------------------------------------------------------------------
@@ -401,7 +460,7 @@ public:
 
   //--------------------------------------------------------------------------------------------------------------------
 
-  virtual void activate() override {
+  virtual void activate(int, int) override {
     owner->change_to_folder(shared_from_this());
   }
 
@@ -478,7 +537,7 @@ public:
   /**
    * Separate tile drawing for the special back tile (to return from a folder).
    */
-  virtual void draw_tile(cairo_t *cr, bool hot, double alpha, bool for_dragging) override {
+  virtual void draw_tile(cairo_t *cr, bool hot, double alpha, bool for_dragging, bool for_migration = false) override {
     draw_tile_background(cr, hot, alpha, for_dragging);
 
     // Title string.
@@ -506,7 +565,7 @@ public:
 
   //--------------------------------------------------------------------------------------------------------------------
 
-  virtual void activate() override {
+  virtual void activate(int, int) override {
     owner->change_to_folder(std::shared_ptr<FolderEntry>());
   }
 
@@ -623,7 +682,7 @@ base::Rect ConnectionsWelcomeScreen::getAccessibilityBounds() {
 //----------------------------------------------------------------------------------------------------------------------
 
 Accessible* ConnectionsWelcomeScreen::accessibilityHitTest(ssize_t x, ssize_t y) {
-  if (_browseDocButton.bounds.contains(static_cast<double>(x), static_cast<double>(y))) {
+   if (_browseDocButton.bounds.contains(static_cast<double>(x), static_cast<double>(y))) {
     return &_browseDocButton;
   }
 
@@ -694,7 +753,7 @@ void ConnectionsWelcomeScreen::repaint(cairo_t *cr, int areax, int areay, int ar
     pos += 0.25;
   }
 
-  _totalHeight = yoffset + 20;
+  _totalHeight = yoffset + 40;
 
   cairo_restore(cr);
 }
@@ -723,7 +782,143 @@ void ConnectionsWelcomeScreen::updateIcons() {
 
 bool ConnectionsWelcomeScreen::mouse_click(mforms::MouseButton button, int x, int y) {
   if (button == MouseButtonLeft) {
-    HomeAccessibleButton * button = dynamic_cast<HomeAccessibleButton *>(accessibilityHitTest(x, y));
+    HomeAccessibleButton *button = dynamic_cast<HomeAccessibleButton *>(accessibilityHitTest(x, y));
+    if (button != nullptr) {
+      button->accessibilityDoDefaultAction();
+      return true;
+    }
+  }
+  return false;
+}
+
+//----------------- ConnectionsMigrationBanner ---------------------------------------------------------------------------
+
+ConnectionsMigrationBanner::ConnectionsMigrationBanner(HomeScreen *owner) : _owner(owner) {
+  logDebug("Creating Connections Migration Assistant Banner\n");  \
+
+  _closeMigrationAssistantBannerButton.title = "Close MigrationAssistant Screen";
+  _closeMigrationAssistantBannerButton.description = "Close MigrationAssistant Screen";
+  _closeMigrationAssistantBannerButton.defaultHandler = [this]() {
+    _owner->trigger_callback(HomeScreenAction::CloseMigrationAssistantBannerMessage, base::any());
+  };
+
+  _migrationLearnMoreButton.title = "Learn more >";
+  _migrationLearnMoreButton.description = "Open Migration Doc";
+  _migrationLearnMoreButton.defaultHandler = [this]() {
+    _owner->trigger_callback(HomeScreenAction::ActionOpenMigrationDoc, base::any());
+  };
+
+  _closeIcon = nullptr;
+  _migrationBanner = nullptr;
+
+  // Load newsticker if lock file does not exist
+  //std::string userdir = mforms::App::get()->get_user_data_folder();
+  //std::string lockfile = userdir + "/ma_lock_fisrt.lck";
+  //FILE *f = fopen(lockfile.c_str(), "r");
+  //if (!f) {
+  //  _mbNewSticker = Utilities::load_icon("newsticker.png", true);
+  //  _showNewSticker = true;
+  //} else {
+  //  _mbNewSticker = nullptr;
+  //  _showNewSticker = false;
+  //  fclose(f);
+  //}
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+Accessible *ConnectionsMigrationBanner::accessibilityHitTest(ssize_t x, ssize_t y) {
+  if (_closeMigrationAssistantBannerButton.bounds.contains(static_cast<double>(x), static_cast<double>(y))) {
+    return &_closeMigrationAssistantBannerButton;
+  }
+  if (_migrationLearnMoreButton.bounds.contains(static_cast<double>(x), static_cast<double>(y))) {
+    return &_migrationLearnMoreButton;
+  }
+
+  return nullptr;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+ConnectionsMigrationBanner::~ConnectionsMigrationBanner() {
+  deleteSurface(_closeIcon);
+  deleteSurface(_migrationBanner);
+  //deleteSurface(_mbNewSticker);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+base::Size ConnectionsMigrationBanner::getLayoutSize(base::Size proposedSize) {
+  return base::Size(proposedSize.width, _totalHeight); // Height doesn't change. Constant content.
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+void ConnectionsMigrationBanner::repaint(cairo_t *cr, int areax, int areay, int areaw, int areah) {
+  Size size = Utilities::getImageSize(_closeIcon);
+  _closeMigrationAssistantBannerButton.bounds = base::Rect(get_width() - size.width - 8, 8, size.width, size.height);
+
+  cairo_save(cr);
+  mforms::Utilities::paint_icon(cr, _closeIcon, _closeMigrationAssistantBannerButton.bounds.left(),
+                                _closeMigrationAssistantBannerButton.bounds.top());
+
+  int yoffset = 100;
+
+  // draw migration assistant banner, scaled and centered
+  size = Utilities::getImageSize(_migrationBanner);
+  double banner_scale = 0.6;
+  double banner_width = size.width * banner_scale;
+  double banner_height = size.height * banner_scale;
+  double x = (get_width() - banner_width) / 2.0;
+  cairo_save(cr);
+  cairo_translate(cr, floor(x), floor(yoffset));
+  cairo_scale(cr, banner_scale, banner_scale);
+  cairo_set_source_surface(cr, _migrationBanner, 0, 0);
+  cairo_paint(cr);
+  // Place _migrationLearnMoreButton as a dummy clickable area (no text) in bottom right corner of banner with 15px
+  // margin
+  double btn_width = banner_width * 0.20;
+  double btn_height = banner_height * 0.25;
+  double btn_x = floor(x + banner_width - btn_width);
+  double btn_y = floor(yoffset + banner_height - btn_height);
+  _migrationLearnMoreButton.bounds = base::Rect(btn_x, btn_y, btn_width - 15, btn_height - 15);
+
+  // Draw newsticker in the top right corner (centered on corner)
+  //if (_showNewSticker && _mbNewSticker) {
+  //  Size sticker_size = Utilities::getImageSize(_mbNewSticker);
+  //  double sticker_x = x + banner_width;
+  //  double sticker_y = -sticker_size.height / 2.0;
+  //  cairo_save(cr);
+  //  cairo_set_source_surface(cr, _mbNewSticker, sticker_x, sticker_y);
+  //  cairo_paint(cr);
+  //}
+
+  _totalHeight = yoffset + (int)banner_height + 20;
+
+  cairo_restore(cr);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+void ConnectionsMigrationBanner::updateIcons() {
+  cairo_surface_destroy(_closeIcon);
+  if (_owner->isDarkModeActive())
+    _closeIcon = Utilities::load_icon("home_screen_close_dark.png", true);
+  else
+    _closeIcon = Utilities::load_icon("home_screen_close_light.png", true);
+
+  cairo_surface_destroy(_migrationBanner);
+  _migrationBanner = Utilities::load_icon("migration_banner.png", true);
+
+  //cairo_surface_destroy(_mbNewSticker);
+  //_mbNewSticker = Utilities::load_icon("new_sticker.png", true);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+bool ConnectionsMigrationBanner::mouse_click(mforms::MouseButton button, int x, int y) {
+  if (button == MouseButtonLeft) {
+    HomeAccessibleButton *button = dynamic_cast<HomeAccessibleButton *>(accessibilityHitTest(x, y));
     if (button != nullptr) {
       button->accessibilityDoDefaultAction();
       return true;
@@ -734,11 +929,15 @@ bool ConnectionsWelcomeScreen::mouse_click(mforms::MouseButton button, int x, in
 
 //------------------ ConnectionsSection --------------------------------------------------------------------------------
 
-ConnectionsSection::ConnectionsSection(HomeScreen *owner) : HomeScreenSection("sidebar_wb.png"),
-  _search_box(true), _search_text(mforms::SmallSearchEntry), _showWelcomeHeading(true) {
-
+ConnectionsSection::ConnectionsSection(HomeScreen *owner)
+  : HomeScreenSection("sidebar_wb.png"),
+    _search_box(true),
+    _search_text(mforms::SmallSearchEntry),
+    _showWelcomeHeading(true), 
+    _showMigrationAssistantBanner(true) {
   _owner = owner;
   _welcomeScreen = nullptr;
+  _migrationAssistantBanner = nullptr;
   _container = nullptr;
   _connection_context_menu = nullptr;
   _folder_context_menu = nullptr;
@@ -751,6 +950,7 @@ ConnectionsSection::ConnectionsSection(HomeScreen *owner) : HomeScreenSection("s
   _network_icon = nullptr;
   _plus_icon = nullptr;
   _sakila_icon = nullptr;
+
   _user_icon = nullptr;
   _manage_icon = nullptr;
 
@@ -902,8 +1102,12 @@ void ConnectionsSection::updateIcons() {
     _user_icon = mforms::Utilities::load_icon("wb_tile_user_light.png");
   }
 
-  if (_welcomeScreen != nullptr)
+  if (_welcomeScreen != nullptr) {
     _welcomeScreen->updateIcons();
+  }
+  if (_migrationAssistantBanner != nullptr) {
+    _migrationAssistantBanner->updateIcons();
+  }
 }
 
 //------------------------------------------------------------------------------------------------
@@ -924,6 +1128,16 @@ void ConnectionsSection::showWelcomeHeading(bool state) {
 
 //------------------------------------------------------------------------------------------------
 
+void ConnectionsSection::showMigrationBanner(bool state) {
+  _showMigrationAssistantBanner = state;
+  if (_migrationAssistantBanner != nullptr)
+    _migrationAssistantBanner->show(state);
+
+  set_layout_dirty(true);
+}
+
+//------------------------------------------------------------------------------------------------
+
 void ConnectionsSection::on_search_text_changed() {
   std::string filter = _search_text.get_string_value();
   _filtered_connections.clear();
@@ -1194,7 +1408,7 @@ void ConnectionsSection::repaint(cairo_t *cr, int areax, int areay, int areaw, i
       connections[index]->bounds = bounds;
 
       bool draw_hot = connections[index] == _hot_entry;
-      connections[index]->draw_tile(cr, draw_hot, 1.0, false);
+      connections[index]->draw_tile(cr, draw_hot, 1.0, false, _on_migration);
 
       // Draw drop indicator.
       if (static_cast<ssize_t>(index) == _drop_index) {
@@ -1523,7 +1737,7 @@ bool ConnectionsSection::mouse_click(mforms::MouseButton button, int x, int y) {
       }
 
       if (_hot_entry) {
-        _hot_entry->activate();
+          _hot_entry->activate(x, y);
         return true;
       }
 
@@ -1567,8 +1781,9 @@ bool ConnectionsSection::mouse_leave() {
 //------------------------------------------------------------------------------------------------
 
 bool ConnectionsSection::mouse_move(mforms::MouseButton button, int x, int y) {
-
   std::shared_ptr<ConnectionEntry> entry = entry_from_point(x, y);
+  bool was_on_migration = _on_migration;
+  _on_migration = entry && entry->getMigrationButtonBounds().contains(x, y);
 
   if (entry && !_mouse_down_position.empty() && (!_mouse_down_position.contains(x, y))) {
     if (!entry->is_movable()) {
@@ -1589,6 +1804,9 @@ bool ConnectionsSection::mouse_move(mforms::MouseButton button, int x, int y) {
         _hot_entry = entry;
         set_needs_repaint();
         return true;
+      } else if (was_on_migration != _on_migration) {
+        set_needs_repaint();
+        return true;
       }
     }
   }
@@ -2005,6 +2223,17 @@ mforms::View *ConnectionsSection::getContainer() {
     _welcomeScreen->setInternalName("welcomeScreen");
     _welcomeScreen->set_layout_dirty(true);
     _container->add(_welcomeScreen, false, true);
+
+#ifdef HAS_MIGRATION_ASSISTANT
+    _migrationAssistantBanner = mforms::manage(new ConnectionsMigrationBanner(_owner));
+    if (!_showMigrationAssistantBanner) {
+      _migrationAssistantBanner->show(false);
+    }
+    _container->add(_migrationAssistantBanner, false, true);
+#else
+    _migrationAssistantBanner = nullptr;
+#endif
+
     _container->add(this, true, true);
   }
   return _container;
